@@ -225,11 +225,15 @@ struct fbTemplate_st {
      * Template context. Created and owned by the application
      * when the listener calls the fbTemplateCtxCallback_fn.
      */
-    void                *ctx;
+    void                *tmpl_ctx;
     /**
      * Callback to free the ctx pointer when template is freed
      */
-    fbTemplateCtxFree_fn ctx_free;
+    fbTemplateCtxFree2_fn ctx_free;
+    /**
+     * The application's Context pointer for the ctx_free function.
+     */
+    void                *app_ctx;
 };
 
 /**
@@ -366,6 +370,18 @@ gboolean            fbInfoElementCopyToTemplateByName(
     uint16_t            len_override,
     fbInfoElement_t     *tmpl_ie);
 
+/**
+ * fbInfoModelAddAlienElement
+ *
+ * @param model
+ * @param ex_ie
+ * @return info_elemnt
+ *
+ */
+const fbInfoElement_t     *fbInfoModelAddAlienElement(
+    fbInfoModel_t       *model,
+    fbInfoElement_t     *ex_ie);
+
 /**
  * fbTemplateRetain
  *
@@ -424,7 +440,17 @@ fbNewTemplateCallback_fn fbSessionTemplateCallback(
  * @param session
  * @return the Context callback function added to the session
  */
-fbTemplateCtxCallback_fn fbSessionTemplateCtxCallback(
+fbTemplateCtxCallback2_fn fbSessionTemplateCtxCallback(
+    fbSession_t *session);
+
+/**
+ * Return the ctx callback function's application context for a given
+ * session
+ *
+ * @param session
+ * @return the Application context pointer added to the session
+ */
+void *fbSessionTemplateCtxCallbackAppCtx(
     fbSession_t *session);
 
 /**
