@@ -1733,8 +1733,9 @@ void            fbCollectorFree(
     collector->cotransClose(collector);
     fbCollectorClose(collector);
 #if HAVE_SPREAD
-    if (collector->coclose == fbCollectorSpreadClose)
+    if (collector->coclose == fbCollectorSpreadClose) {
         fbConnSpreadFree( collector->stream.spread );
+    }
 #endif
     while (collector->udp_tail) {
         fbCollectorFreeUDPSpec(collector, collector->udp_tail);
