@@ -3,7 +3,7 @@
  ** IPFIX Transport Session state container
  **
  ** ------------------------------------------------------------------------
- ** Copyright (C) 2006-2014 Carnegie Mellon University. All Rights Reserved.
+ ** Copyright (C) 2006-2015 Carnegie Mellon University. All Rights Reserved.
  ** ------------------------------------------------------------------------
  ** Authors: Brian Trammell
  ** ------------------------------------------------------------------------
@@ -356,8 +356,9 @@ uint16_t    fbSessionLookupTemplatePair(
 static void     fbSessionFreeOneTemplate(
     void            *vtid __attribute__((unused)),
     fbTemplate_t    *tmpl,
-    fbSession_t     *session __attribute__((unused)))
+    fbSession_t     *session)
 {
+    fBufRemoveTemplateTcplan(session->tdyn_buf, tmpl);
     fbTemplateRelease(tmpl);
 }
 
@@ -843,7 +844,8 @@ uint16_t        fbSessionAddTemplate(
             tid = next_int_tid--;
         } else {
             if (next_ext_tid == 0) next_ext_tid = FB_TID_MIN_DATA;
-            while (fbSessionGetTemplate(session, internal, next_ext_tid, NULL))             {
+            while (fbSessionGetTemplate(session, internal, next_ext_tid, NULL))
+            {
                 next_ext_tid++;
                 if (next_ext_tid == 0) next_ext_tid = FB_TID_MIN_DATA;
             }
@@ -956,7 +958,6 @@ fbTemplate_t    *fbSessionGetTemplate(
     if (!internal)
         pthread_mutex_unlock( &session->ext_ttab_wlock );
 #endif
-
     /* Check for missing template */
     if (!tmpl) {
         if (internal) {
