@@ -7,7 +7,7 @@
  * within the fixbuf structure
  *
  * ------------------------------------------------------------------------
- * Copyright (C) 2008-2014 Carnegie Mellon University. All Rights Reserved.
+ * Copyright (C) 2008-2015 Carnegie Mellon University. All Rights Reserved.
  * ------------------------------------------------------------------------
  * Authors: Emily Sarneso <ecoff@cert.org>
  * ------------------------------------------------------------------------
@@ -70,6 +70,7 @@
 
 #include "fbcollector.h"
 #include <sys/time.h>
+#include <pthread.h>
 
 #ident "$Id$"
 
@@ -470,6 +471,7 @@ struct fbCollectorSFlowState_st {
     fBuf_t                        *fbuf;
     uint8_t                       *ipfixBuffer;
     GHashTable                    *domainHash;
+    pthread_mutex_t               ts_lock;
 };
 
 
@@ -1316,6 +1318,8 @@ static gboolean     fbCollectorPostProcSFlow(
     memset(&sflowrec, 0, sizeof(sflowrec));
     memset(&sflowctr, 0, sizeof(sflowctr));
 
+    pthread_mutex_lock(&transState->ts_lock);
+
     memset(transState->ipfixBuffer, 0, 65535);
 
     if (!transState->fbuf) {
@@ -1328,6 +1332,7 @@ static gboolean     fbCollectorPostProcSFlow(
     sfexp = sflowAllocExporter(transState->ipfixBuffer, transState->fbuf, err);
 
     if (!sfexp) {
+        pthread_mutex_unlock(&transState->ts_lock);
         return FALSE;
     }
 
@@ -1375,6 +1380,7 @@ static gboolean     fbCollectorPostProcSFlow(
     if (newbuffer) {
 
         if (!fbSessionExportTemplates(transState->exsession, err)) {
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
         }
 
@@ -1387,6 +1393,7 @@ static gboolean     fbCollectorPostProcSFlow(
 #endif
         memcpy(dataBuf, transState->ipfixBuffer, msglen);
         *bufLen = msglen;
+        pthread_mutex_unlock(&transState->ts_lock);
         return TRUE;
 
     }
@@ -1448,6 +1455,7 @@ static gboolean     fbCollectorPostProcSFlow(
             g_set_error(err, FB_ERROR_DOMAIN, FB_ERROR_SFLOW,
                         "Buffer too small for Sample Header");
             currentSession->sflowSeqNum++;
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
         }
 
@@ -1469,6 +1477,7 @@ static gboolean     fbCollectorPostProcSFlow(
                         "Invalid sFlow enterprise number (%u)",
                         enterprise);
             currentSession->sflowSeqNum++;
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
         }
 
@@ -1477,6 +1486,7 @@ static gboolean     fbCollectorPostProcSFlow(
                         "Buffer too small for sample length (%u)",
                         sampleLength);
             currentSession->sflowSeqNum++;
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
         }
 
@@ -1555,12 +1565,14 @@ static gboolean     fbCollectorPostProcSFlow(
             g_set_error(err, FB_ERROR_DOMAIN, FB_ERROR_SFLOW,
                         "Invalid sFlow Format (%d)", format);
             currentSession->sflowSeqNum++;
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
         }
 
         if (!flows && !counters) {
             /* error ocurred */
             currentSession->sflowSeqNum++;
+            pthread_mutex_unlock(&transState->ts_lock);
             return FALSE;
         }
 
@@ -1568,12 +1580,16 @@ static gboolean     fbCollectorPostProcSFlow(
 
     }
 
+
+    /* there is possibly extra filling at the end of the packet */
+    /* warn and ignore! */
+
     if (msgParsed) {
-        g_set_error(err, FB_ERROR_DOMAIN, FB_ERROR_SFLOW,
+#ifndef FB_SUPPRESS_LOGS
+        g_log(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
                     "sFlow Record Length Mismatch: (buffer has "
                     "%zu, leftover %zu)", *bufLen, msgParsed);
-        currentSession->sflowSeqNum++;
-        return FALSE;
+#endif
     }
 
     /* increment the sequence number for the netflow side */
@@ -1588,6 +1604,8 @@ static gboolean     fbCollectorPostProcSFlow(
 
     fbExporterClose(sfexp);
 
+    pthread_mutex_unlock(&transState->ts_lock);
+
     return TRUE;
 }
 
@@ -1628,6 +1646,8 @@ static void         fbCollectorTransCloseSFlow(
     g_hash_table_destroy(transState->domainHash);
     transState->domainHash = NULL;
 
+    pthread_mutex_destroy(&transState->ts_lock);
+
     if (NULL != collector->translatorState) {
         g_slice_free1(sizeof(struct fbCollectorSFlowState_st),
                       collector->translatorState);
@@ -1659,10 +1679,13 @@ static void fbCollectorTimeOutSessionSFlow(
         return;
     }
 
+    pthread_mutex_lock(&transState->ts_lock);
+
     sfsession = g_hash_table_lookup(transState->domainHash, session);
 
     if (sfsession == NULL) {
         /* don't need to free! */
+        pthread_mutex_unlock(&transState->ts_lock);
         return;
     }
 
@@ -1674,7 +1697,7 @@ static void fbCollectorTimeOutSessionSFlow(
         transState->session = NULL;
     }
 
-    return;
+    pthread_mutex_unlock(&transState->ts_lock);
 
 }
 
@@ -1766,10 +1789,12 @@ gboolean    fbCollectorSetSFlowTranslator(
     sflowState->exsession = sfsess;
     sflowState->model = model;
     sflowState->ipfixBuffer = g_slice_alloc0(65535);
+    pthread_mutex_init(&sflowState->ts_lock, NULL);
 
     return fbCollectorSetTranslator(collector, fbCollectorPostProcSFlow,
         fbCollectorDecodeSFlowMsgVL, fbCollectorMessageHeaderSFlow,
-        fbCollectorTransCloseSFlow, fbCollectorTimeOutSessionSFlow, sflowState, err);
+        fbCollectorTransCloseSFlow, fbCollectorTimeOutSessionSFlow,
+                                    sflowState, err);
 
 }
 
@@ -1800,6 +1825,8 @@ uint32_t fbCollectorGetSFlowMissed(
     struct fbCollectorSFlowState_st     *ts = NULL;
     fbUDPConnSpec_t                     *udp = NULL;
     fbSession_t                         *session = NULL;
+    fbCollectorSFlowSession_t           *sfsession = NULL;
+    uint32_t                            missed = 0;
 
     if (!collector) {
         return 0;
@@ -1837,13 +1864,20 @@ uint32_t fbCollectorGetSFlowMissed(
         return 0;
     }
 
+    pthread_mutex_lock(&ts->ts_lock);
+
     if (ts->cosession != session) {
         /* lookup template Hash Table per Domain */
-        ts->session = g_hash_table_lookup(ts->domainHash, session);
-        if (ts->session == NULL) {
-            return 0;
-        }
+        sfsession = g_hash_table_lookup(ts->domainHash, session);
+    } else {
+        sfsession = ts->session;
     }
 
-    return ts->session->sflowMissed;
+    if (sfsession) {
+        missed = sfsession->sflowMissed;
+    }
+
+    pthread_mutex_unlock(&ts->ts_lock);
+
+    return missed;
 }
